@@ -23,18 +23,19 @@
 #include <config.h>
 
 #include <bonobo/bonobo-main.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include <string.h>
 
 #include "yelp-cache.h"
 #include "yelp-window.h"
-#include "yelp-section.h"
-#include "yelp-scrollkeeper.h"
-#include "yelp-man.h"
-#include "yelp-info.h"
+#include "yelp-settings.h"
+#include "yelp-pager.h"
 #include "yelp-toc-pager.h"
 #include "yelp-base.h"
 
+gboolean main_running;
+
 struct _YelpBasePriv {
 	GNode  *toc_tree;
 
@@ -45,6 +46,7 @@ struct _YelpBasePriv {
 static void           yelp_base_init                (YelpBase       *base);
 static void           yelp_base_class_init          (YelpBaseClass  *klass);
 static void           yelp_base_new_window_cb       (YelpWindow     *window,
+						     const gchar    *uri,
 						     YelpBase       *base);
 static void           yelp_base_window_finalized_cb (YelpBase       *base,
 						     YelpWindow     *window);
@@ -74,7 +76,8 @@ impl_Yelp_getWindows (PortableServer_Servant  servant,
 	GNOME_Yelp_WindowList *list;
 	gint                   len, i;
 	GSList                *node;
-	YelpURI               *uri;
+	YelpDocInfo           *doc_info;
+	gchar                 *uri;
 	
 	base = YELP_BASE (bonobo_object (servant));
 	priv = base->priv;
@@ -88,12 +91,10 @@ impl_Yelp_getWindows (PortableServer_Servant  servant,
 	CORBA_sequence_set_release (list, CORBA_TRUE);
 	
 	for (node = priv->windows, i = 0; node; node = node->next, i++) {
-		gchar *str_uri;
-
-		uri = yelp_window_get_current_uri (YELP_WINDOW (node->data));
-		str_uri = yelp_uri_to_string (uri);
-		list->_buffer[i] = CORBA_string_dup (str_uri);
-		g_free (str_uri);
+		doc_info = yelp_window_get_doc_info (YELP_WINDOW (node->data));
+		uri = yelp_doc_info_get_uri (doc_info, NULL, YELP_URI_TYPE_ANY);
+		list->_buffer[i] = CORBA_string_dup (uri);
+		g_free (uri);
 	}
 	
 	return list;
@@ -112,30 +113,34 @@ yelp_base_init (YelpBase *base)
         base->priv     = priv;
 
 	yelp_cache_init ();
+	yelp_settings_init ();
 }
 
 static void
 yelp_base_class_init (YelpBaseClass *klass)
 {
 	POA_GNOME_Yelp__epv *epv = &klass->epv;
-	
+
 	parent_class = gtk_type_class (PARENT_TYPE);
 
 	epv->newWindow  = impl_Yelp_newWindow;
 	epv->getWindows = impl_Yelp_getWindows;
+
+	main_running = TRUE;
 }
 
 static void
-yelp_base_new_window_cb (YelpWindow *window, YelpBase *base)
+yelp_base_new_window_cb (YelpWindow *window, const gchar *uri,
+			 YelpBase *base)
 {
 	GtkWidget *new_window;
 	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 	g_return_if_fail (YELP_IS_BASE (base));
 	
-	new_window = yelp_base_new_window (base, NULL);
+	new_window = yelp_base_new_window (base, uri);
 	
-	gtk_widget_show_all (new_window);
+	gtk_widget_show (new_window);
 }
 
 static void
@@ -144,12 +149,13 @@ yelp_base_window_finalized_cb (YelpBase *base, YelpWindow *window)
 	YelpBasePriv *priv;
 	
 	g_return_if_fail (YELP_IS_BASE (base));
-	
+
 	priv = base->priv;
 	
 	priv->windows = g_slist_remove (priv->windows, window);
 
 	if (g_slist_length (priv->windows) == 0) {
+		main_running = FALSE;
 		bonobo_main_quit ();
 	}
 }
@@ -177,10 +183,10 @@ yelp_base_new (void)
 }
 
 GtkWidget *
-yelp_base_new_window (YelpBase *base, const gchar *str_uri)
+yelp_base_new_window (YelpBase *base, const gchar *uri)
 {
 	YelpBasePriv *priv;
-	YelpURI      *uri;
+	gchar        *str;
 	GtkWidget    *window;
         
         g_return_val_if_fail (YELP_IS_BASE (base), NULL);
@@ -199,17 +205,22 @@ yelp_base_new_window (YelpBase *base, const gchar *str_uri)
 			  G_CALLBACK (yelp_base_new_window_cb),
 			  base);
 
+	gtk_widget_show (window);
 
-	gtk_widget_show_all (window);
+	if (uri && strcmp (uri, "")) {
+		gchar *dir = g_get_current_dir ();
+		gchar *dirs = g_strconcat ("file://", dir, "/", NULL);
+		str = gnome_vfs_uri_make_full_from_relative (dirs, uri);
+		g_free (dirs);
+		g_free (dir);
+	} else {
+		str = g_strdup ("x-yelp-toc:");
+	}
 
-	if (str_uri && strcmp (str_uri, ""))
-		uri = yelp_uri_new (str_uri);
-	else
-		uri = yelp_uri_new ("toc:");
-
-	yelp_window_open_uri (YELP_WINDOW (window), uri);
+	yelp_window_load (YELP_WINDOW (window), str);
+	g_free (str);
 
 	return window;
 }
 
-BONOBO_TYPE_FUNC_FULL (YelpBase, GNOME_Yelp, PARENT_TYPE, yelp_base);
+BONOBO_TYPE_FUNC_FULL (YelpBase, GNOME_Yelp, PARENT_TYPE, yelp_base)
