@@ -39,7 +39,13 @@
 #include "yelp-error.h"
 #include "yelp-db-pager.h"
 #include "yelp-toc-pager.h"
-#include "yelp-theme.h"
+#include "yelp-settings.h"
+
+#ifdef YELP_DEBUG
+#define d(x) x
+#else
+#define d(x)
+#endif
 
 #define YELP_NAMESPACE "http://www.gnome.org/yelp/ns"
 
@@ -49,7 +55,10 @@
 #define BOOK_CHUNK_DEPTH 2
 #define ARTICLE_CHUNK_DEPTH 1
 
-#define d(x)
+#define EVENTS_PENDING while (yelp_pager_get_state (pager) <= YELP_PAGER_STATE_RUNNING && gtk_events_pending ()) gtk_main_iteration ();
+#define CANCEL_CHECK if (!main_running || yelp_pager_get_state (pager) >= YELP_PAGER_STATE_ERROR) goto done;
+
+extern gboolean main_running;
 
 struct _YelpDBPagerPriv {
     GtkTreeModel   *sects;
@@ -57,6 +66,12 @@ struct _YelpDBPagerPriv {
     GHashTable     *frags_hash;
 
     gchar          *root_id;
+
+    xmlDocPtr               inputDoc;
+    xmlDocPtr               outputDoc;
+    xmlParserCtxtPtr        parserCtxt;
+    xsltStylesheetPtr       stylesheet;
+    xsltTransformContextPtr transformContext;
 };
 
 typedef struct _DBWalker DBWalker;
@@ -77,14 +92,14 @@ static void          db_pager_class_init   (YelpDBPagerClass *klass);
 static void          db_pager_init         (YelpDBPager      *pager);
 static void          db_pager_dispose      (GObject          *gobject);
 
-void                 db_pager_error        (YelpPager        *pager);
-void                 db_pager_cancel       (YelpPager        *pager);
-void                 db_pager_finish       (YelpPager        *pager);
+static void          db_pager_error        (YelpPager        *pager);
+static void          db_pager_cancel       (YelpPager        *pager);
+static void          db_pager_finish       (YelpPager        *pager);
 
 gboolean             db_pager_process      (YelpPager        *pager);
-gchar *              db_pager_resolve_uri  (YelpPager        *pager,
-					    YelpURI          *uri);
-const GtkTreeModel * db_pager_get_sections (YelpPager        *pager);
+const gchar *        db_pager_resolve_frag (YelpPager        *pager,
+					    const gchar      *frag_id);
+GtkTreeModel *       db_pager_get_sections (YelpPager        *pager);
 
 static void          walker_walk_xml       (DBWalker         *walker);
 static gboolean      walker_is_chunk       (DBWalker         *walker);
@@ -143,7 +158,7 @@ db_pager_class_init (YelpDBPagerClass *klass)
     pager_class->finish       = db_pager_finish;
 
     pager_class->process      = db_pager_process;
-    pager_class->resolve_uri  = db_pager_resolve_uri;
+    pager_class->resolve_frag = db_pager_resolve_frag;
     pager_class->get_sections = db_pager_get_sections;
 }
 
@@ -180,14 +195,14 @@ db_pager_dispose (GObject *object)
 /******************************************************************************/
 
 YelpPager *
-yelp_db_pager_new (YelpURI *uri)
+yelp_db_pager_new (YelpDocInfo *doc_info)
 {
     YelpDBPager *pager;
 
-    g_return_val_if_fail (uri != NULL, NULL);
+    g_return_val_if_fail (doc_info != NULL, NULL);
 
     pager = (YelpDBPager *) g_object_new (YELP_TYPE_DB_PAGER,
-					  "uri", uri,
+					  "document-info", doc_info,
 					  NULL);
 
     pager->priv->sects =
@@ -199,68 +214,62 @@ yelp_db_pager_new (YelpURI *uri)
 gboolean
 db_pager_process (YelpPager *pager)
 {
-    YelpURI   *uri = yelp_pager_get_uri (pager);
-    gchar     *path;
-    DBWalker  *walker;
-    xmlChar   *id;
-    GError    *error = NULL;
+    YelpDBPagerPriv *priv;
+    YelpDocInfo     *doc_info;
+    gchar           *filename;
+    gint  params_i = 0;
+    gint  i;
 
-    xmlDocPtr               doc, newdoc;
-    xmlParserCtxtPtr        ctxt;
-    xsltStylesheetPtr       stylesheet;
-    xsltTransformContextPtr tctxt;
+    DBWalker    *walker;
+    xmlChar     *id;
+    GError      *error = NULL;
 
-    gchar *p_doc_name, *p_doc_path;
+    GtkIconInfo *icon_info;
+    gchar *icon_file;
+    gchar *icons[YELP_NUM_ICONS];
 
-    YelpDBPagerPriv *priv = YELP_DB_PAGER (pager)->priv;
-
-    gchar        *doc_name;
-    gchar        *doc_path;
     const gchar  *params[40];
-    gint i = 0;
+
+    d (g_print ("db_pager_process\n"));
+
+    doc_info = yelp_pager_get_doc_info (pager);
 
     g_return_val_if_fail (pager != NULL, FALSE);
     g_return_val_if_fail (YELP_IS_DB_PAGER (pager), FALSE);
+    priv = YELP_DB_PAGER (pager)->priv;
 
-    path = gnome_vfs_uri_to_string (uri->uri,
-				    GNOME_VFS_URI_HIDE_USER_NAME           |
-				    GNOME_VFS_URI_HIDE_PASSWORD            |
-				    GNOME_VFS_URI_HIDE_HOST_NAME           |
-				    GNOME_VFS_URI_HIDE_HOST_PORT           |
-				    GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD     |
-				    GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER );
+    if (yelp_pager_get_state (pager) >= YELP_PAGER_STATE_ERROR)
+	goto done;
+
+    filename = yelp_doc_info_get_filename (doc_info);
 
     g_object_ref (pager);
-
     yelp_toc_pager_pause (yelp_toc_pager_get ());
 
-    ctxt = xmlNewParserCtxt ();
-    doc = xmlCtxtReadFile (ctxt,
-			   (const char *) path,
-			   NULL,
-			   XML_PARSE_DTDLOAD  |
-			   XML_PARSE_NOCDATA  |
-			   XML_PARSE_NOENT    |
-			   XML_PARSE_NONET    );
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_PARSING);
+    g_signal_emit_by_name (pager, "parse");
 
-    if (doc == NULL) {
+    priv->parserCtxt = xmlNewParserCtxt ();
+    priv->inputDoc   = xmlCtxtReadFile (priv->parserCtxt,
+					(const char *) filename, NULL,
+					XML_PARSE_DTDLOAD | XML_PARSE_NOCDATA |
+					XML_PARSE_NOENT   | XML_PARSE_NONET   );
+    if (priv->inputDoc == NULL) {
 	yelp_set_error (&error, YELP_ERROR_NO_DOC);
 	yelp_pager_error (pager, error);
-	return FALSE;
+	goto done;
     }
 
-    xmlXIncludeProcessFlags (doc,
-			     XML_PARSE_DTDLOAD  |
-			     XML_PARSE_NOCDATA  |
-			     XML_PARSE_NOENT    |
-			     XML_PARSE_NONET    );
+    xmlXIncludeProcessFlags (priv->inputDoc,
+			     XML_PARSE_DTDLOAD | XML_PARSE_NOCDATA |
+			     XML_PARSE_NOENT   | XML_PARSE_NONET   );
 
     walker = g_new0 (DBWalker, 1);
     walker->pager     = YELP_DB_PAGER (pager);
-    walker->doc       = doc;
+    walker->doc       = priv->inputDoc;
     walker->cur       = xmlDocGetRootElement (walker->doc);
 
-    if (!xmlStrcmp (xmlDocGetRootElement (doc)->name, BAD_CAST "book"))
+    if (!xmlStrcmp (xmlDocGetRootElement (priv->inputDoc)->name, BAD_CAST "book"))
 	walker->max_depth = BOOK_CHUNK_DEPTH;
     else
 	walker->max_depth = ARTICLE_CHUNK_DEPTH;
@@ -271,109 +280,167 @@ db_pager_process (YelpPager *pager)
     else
 	priv->root_id = g_strdup ("index");
 
-    while (gtk_events_pending ())
-	gtk_main_iteration ();
+    EVENTS_PENDING;
+    CANCEL_CHECK;
 
     walker_walk_xml (walker);
 
-    yelp_pager_set_state  (pager, YELP_PAGER_STATE_CONTENTS);
-    g_signal_emit_by_name (pager, "contents");
+    CANCEL_CHECK;
 
-    while (gtk_events_pending ())
-	gtk_main_iteration ();
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_RUNNING);
+    g_signal_emit_by_name (pager, "start");
 
-    doc_name = gnome_vfs_uri_extract_short_name (uri->uri);
-    doc_path = gnome_vfs_uri_extract_dirname (uri->uri);
-    p_doc_name = g_strconcat("\"", doc_name, "\"", NULL);
-    p_doc_path = g_strconcat("\"file://", doc_path, "/\"", NULL);
+    EVENTS_PENDING;
+    CANCEL_CHECK;
 
-    params[i++] = "doc_name";
-    params[i++] = p_doc_name;
-    params[i++] = "doc_path";
-    params[i++] = p_doc_path;
-    params[i++] = "stylesheet_path";
-    params[i++] = "\"file://" DB_STYLESHEET_PATH "/\"";
-    params[i++] = "html_extension";
-    params[i++] = "\"\"";
-    params[i++] = "resolve_xref_chunk";
-    params[i++] = "0";
-    params[i++] = "mediaobject_path";
-    params[i++] = p_doc_path;
-    params[i++] = "color_gray_background";
-    params[i++] = yelp_theme_get_gray_background ();
-    params[i++] = "color_gray_border";
-    params[i++] = yelp_theme_get_gray_border ();
-    params[i++] = "admon_graphics_path";
-    params[i++] = "\"file://" DATADIR "/yelp/icons/\"";
-    params[i++] = NULL;
+    for (i = 0; i < YELP_NUM_ICONS; i++) {
+	icon_info = yelp_settings_get_icon (i);
+	if (icon_info) {
+	    icon_file = (gchar *) gtk_icon_info_get_filename (icon_info);
+	    if (icon_file)
+		icons[i] = g_strdup_printf ("\"%s\"", icon_file);
+	    else 
+		icons[i] = g_strdup ("\"\"");
+	    gtk_icon_info_free (icon_info);
+	} else {
+	    icons[i] = g_strdup ("\"\"");
+	}
 
-    stylesheet = xsltParseStylesheetFile (DB_STYLESHEET);
-    tctxt      = xsltNewTransformContext (stylesheet,
-					  doc);
-    tctxt->_private = pager;
-    xsltRegisterExtElement (tctxt,
+	switch (i) {
+	case YELP_ICON_BLOCKQUOTE:
+	    params[params_i++] = "yelp.image.blockquote";
+	    break;
+	case YELP_ICON_CAUTION:
+	    params[params_i++] = "yelp.image.caution";
+	    break;
+	case YELP_ICON_IMPORTANT:
+	    params[params_i++] = "yelp.image.important";
+	    break;
+	case YELP_ICON_NOTE:
+	    params[params_i++] = "yelp.image.note";
+	    break;
+	case YELP_ICON_PROGRAMLISTING:
+	    params[params_i++] = "yelp.image.programlisting";
+	    break;
+	case YELP_ICON_TIP:
+	    params[params_i++] = "yelp.image.tip";
+	    break;
+	case YELP_ICON_WARNING:
+	    params[params_i++] = "yelp.image.warning";
+	    break;
+	default:
+	    g_assert_not_reached ();
+	}
+	params[params_i++] = icons[i];
+    }
+
+    params[params_i++] = "stylesheet_path";
+    params[params_i++] = "\"file://" DB_STYLESHEET_PATH "/\"";
+    params[params_i++] = "html_extension";
+    params[params_i++] = "\"\"";
+    params[params_i++] = "resolve_xref_chunk";
+    params[params_i++] = "0";
+    params[params_i++] = "admon_graphics_path";
+    params[params_i++] = "\"file://" DATADIR "/yelp/icons/\"";
+    params[params_i++] = NULL;
+
+    priv->stylesheet = xsltParseStylesheetFile (DB_STYLESHEET);
+    if (!priv->stylesheet) {
+	yelp_set_error (&error, YELP_ERROR_PROC);
+	yelp_pager_error (pager, error);
+	goto done;
+    }
+
+    priv->transformContext = xsltNewTransformContext (priv->stylesheet,
+						      priv->inputDoc);
+    if (!priv->transformContext) {
+	yelp_set_error (&error, YELP_ERROR_PROC);
+	yelp_pager_error (pager, error);
+	goto done;
+    }
+
+    priv->transformContext->_private = pager;
+    xsltRegisterExtElement (priv->transformContext,
 			    "document",
 			    YELP_NAMESPACE,
 			    (xsltTransformFunction) xslt_yelp_document);
-    xsltRegisterExtElement (tctxt,
+    xsltRegisterExtElement (priv->transformContext,
 			    "cache",
 			    YELP_NAMESPACE,
 			    (xsltTransformFunction) xslt_yelp_cache);
 
-    while (gtk_events_pending ())
-	gtk_main_iteration ();
+    EVENTS_PENDING;
+    CANCEL_CHECK;
 
-    newdoc = xsltApplyStylesheetUser (stylesheet,
-				      doc,
-				      params,
-				      NULL, NULL,
-				      tctxt);
-    xmlFreeDoc (newdoc);
-
-    g_free (path);
-    g_free (walker);
-    g_free (p_doc_name);
-    g_free (p_doc_path);
-    g_free (doc_name);
-    g_free (doc_path);
-
-    xmlFreeDoc (doc);
-    xsltFreeStylesheet (stylesheet);
-    xmlFreeParserCtxt (ctxt);
-    xsltFreeTransformContext (tctxt);
-
-    yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
+    priv->outputDoc = xsltApplyStylesheetUser (priv->stylesheet,
+					       priv->inputDoc,
+					       params, NULL, NULL,
+					       priv->transformContext);
+    CANCEL_CHECK;
     g_signal_emit_by_name (pager, "finish");
 
+ done:
+    g_free (filename);
+    g_free (walker);
+
+    if (priv->inputDoc) {
+	xmlFreeDoc (priv->inputDoc);
+	priv->inputDoc = NULL;
+    }
+    if (priv->outputDoc) {
+	xmlFreeDoc (priv->outputDoc);
+	priv->outputDoc = NULL;
+    }
+    if (priv->parserCtxt) {
+	xmlFreeParserCtxt (priv->parserCtxt);
+	priv->parserCtxt = NULL;
+    }
+    if (priv->stylesheet) {
+	xsltFreeStylesheet (priv->stylesheet);
+	priv->stylesheet = NULL;
+    }
+    if (priv->transformContext) {
+	xsltFreeTransformContext (priv->transformContext);
+	priv->transformContext = NULL;
+    }
+
     g_object_unref (pager);
 
     return FALSE;
 }
 
-void
+static void
 db_pager_error (YelpPager   *pager)
 {
-    yelp_toc_pager_unpause (yelp_toc_pager_get ());
+    d (g_print ("db_pager_error\n"));
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_ERROR);
+    if (yelp_pager_get_state (pager) <= YELP_PAGER_STATE_RUNNING)
+	yelp_toc_pager_unpause (yelp_toc_pager_get ());
 }
 
-void
+static void
 db_pager_cancel (YelpPager *pager)
 {
-    yelp_toc_pager_unpause (yelp_toc_pager_get ());
+    d (g_print ("db_pager_cancel\n"));
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_INVALID);
+    if (yelp_pager_get_state (pager) <= YELP_PAGER_STATE_RUNNING)
+	yelp_toc_pager_unpause (yelp_toc_pager_get ());
     // FIXME: actually cancel
 }
 
-void
+static void
 db_pager_finish (YelpPager   *pager)
 {
-    yelp_toc_pager_unpause (yelp_toc_pager_get ());
+    d (g_print ("db_pager_finish\n"));
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
+    if (yelp_pager_get_state (pager) <= YELP_PAGER_STATE_RUNNING)
+	yelp_toc_pager_unpause (yelp_toc_pager_get ());
 }
 
-gchar *
-db_pager_resolve_uri (YelpPager *pager, YelpURI *uri)
+const gchar *
+db_pager_resolve_frag (YelpPager *pager, const gchar *frag_id)
 {
     YelpDBPager  *db_pager;
-    const gchar  *frag_id;
     gchar        *page_id = NULL;
 
     g_return_val_if_fail (pager != NULL, NULL);
@@ -381,20 +448,16 @@ db_pager_resolve_uri (YelpPager *pager, YelpURI *uri)
 
     db_pager = YELP_DB_PAGER (pager);
 
-    frag_id = gnome_vfs_uri_get_fragment_identifier (uri->uri);
-
     if (frag_id)
 	page_id = g_hash_table_lookup (db_pager->priv->frags_hash,
 				       frag_id);
-    else if (db_pager->priv->root_id)
-	page_id = db_pager->priv->root_id;
     else
-	page_id = "index";
+	page_id = db_pager->priv->root_id;
 
-    return g_strdup (page_id);
+    return (const gchar *) page_id;
 }
 
-const GtkTreeModel *
+GtkTreeModel *
 db_pager_get_sections (YelpPager *pager)
 {
     g_return_val_if_fail (pager != NULL, NULL);
@@ -418,18 +481,19 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
     YelpPager *pager;
     xsltStylesheetPtr style = NULL;
     const char *old_outfile;
-    xmlDocPtr   new_doc, old_doc;
+    xmlDocPtr   new_doc = NULL;
+    xmlDocPtr   old_doc;
     xmlNodePtr  old_insert;
     xmlNodePtr  cur;
 
     if (!ctxt || !node || !inst || !comp)
 	return;
 
-    while (gtk_events_pending ())
-	gtk_main_iteration ();
-
     pager = (YelpPager *) ctxt->_private;
 
+    EVENTS_PENDING;
+    CANCEL_CHECK;
+
     page_id = xsltEvalAttrValueTemplate (ctxt, inst,
 					  (const xmlChar *) "href",
 					  NULL);
@@ -438,8 +502,7 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 			    _("No href attribute found on yelp:document"));
 	error = NULL;
 	yelp_pager_error (pager, error);
-
-	return;
+	goto done;
     }
 
     old_outfile = ctxt->outputFile;
@@ -453,8 +516,7 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 			    _("Out of memory"));
 	error = NULL;
 	yelp_pager_error (pager, error);
-
-	return;
+	goto done;
     }
 
     style->omitXmlDeclaration = TRUE;
@@ -469,6 +531,8 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 
     xsltApplyOneTemplate (ctxt, node, inst->children, NULL, NULL);
 
+    CANCEL_CHECK;
+
     xsltSaveResultToString (&page_buf, &buf_size, new_doc, style);
 
     ctxt->outputFile = old_outfile;
@@ -477,14 +541,16 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 
     page_title = xml_get_title (node);
 
+    CANCEL_CHECK;
+
     if (!page_title)
 	page_title = g_strdup ("FIXME");
 
     page = g_new0 (YelpPage, 1);
 
-    page->id    = page_id;
-    page->title = page_title;
-    page->chunk = page_buf;
+    page->page_id  = page_id;
+    page->title    = page_title;
+    page->contents = page_buf;
 
     cur = xmlDocGetRootElement (new_doc);
     for (cur = cur->children; cur; cur = cur->next) {
@@ -494,11 +560,11 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 		    xmlChar *rel = xmlGetProp (cur, "rel");
 
 		    if (!xmlStrcmp (rel, (xmlChar *) "Previous"))
-			page->prev = xmlGetProp (cur, "href");
+			page->prev_id = xmlGetProp (cur, "href");
 		    else if (!xmlStrcmp (rel, (xmlChar *) "Next"))
-			page->next = xmlGetProp (cur, "href");
+			page->next_id = xmlGetProp (cur, "href");
 		    else if (!xmlStrcmp (rel, (xmlChar *) "Top"))
-			page->toc = xmlGetProp (cur, "href");
+			page->toc_id = xmlGetProp (cur, "href");
 
 		    xmlFree (rel);
 		}
@@ -507,14 +573,19 @@ xslt_yelp_document (xsltTransformContextPtr ctxt,
 	}
     }
 
+    CANCEL_CHECK;
+
     yelp_pager_add_page (pager, page);
     g_signal_emit_by_name (pager, "page", page_id);
 
-    while (gtk_events_pending ())
-	gtk_main_iteration ();
+    EVENTS_PENDING;
+    CANCEL_CHECK;
 
-    xmlFreeDoc (new_doc);
-    xsltFreeStylesheet (style);
+ done:
+    if (new_doc)
+	xmlFreeDoc (new_doc);
+    if (style)
+	xsltFreeStylesheet (style);
 }
 
 void
@@ -527,6 +598,7 @@ xslt_yelp_cache (xsltTransformContextPtr ctxt,
 
     while (gtk_events_pending ())
 	gtk_main_iteration ();
+    // FIXME : check for cancel
 }
 
 static void
@@ -556,7 +628,7 @@ walker_walk_xml (DBWalker *walker)
 
     id = xmlGetProp (walker->cur, "id");
     if (!id && walker->cur->parent->type == XML_DOCUMENT_NODE) {
-	id = xmlStrdup ("index");
+	id = xmlStrdup ("__yelp_toc");
     }
 
     if (walker_is_chunk (walker)) {
@@ -596,6 +668,7 @@ walker_walk_xml (DBWalker *walker)
 
     while (gtk_events_pending ())
 	gtk_main_iteration ();
+    // FIXME : check for cancel
 
     old_cur = walker->cur;
     walker->depth++;
