@@ -27,11 +27,12 @@
 #include <gtk/gtktypeutils.h>
 #include <gtk/gtkmarshal.h>
 
-#define YELP_TYPE_HTML        (yelp_html_get_type ())
-#define YELP_HTML(o)          (GTK_CHECK_CAST ((o), YELP_TYPE_HTML, YelpHtml))
-#define YELP_HTML_CLASS(k)    (GTK_CHECK_FOR_CAST((k), YELP_TYPE_HTML, YelpHtmlClass))
-#define YELP_IS_HTML(o)       (GTK_CHECK_TYPE ((o), YELP_TYPE_HTML))
-#define YELP_IS_HTML_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), YELP_TYPE_HTML))
+#define YELP_TYPE_HTML         (yelp_html_get_type ())
+#define YELP_HTML(o)           (GTK_CHECK_CAST ((o), YELP_TYPE_HTML, YelpHtml))
+#define YELP_HTML_CLASS(k)     (GTK_CHECK_FOR_CAST((k), YELP_TYPE_HTML, YelpHtmlClass))
+#define YELP_IS_HTML(o)        (GTK_CHECK_TYPE ((o), YELP_TYPE_HTML))
+#define YELP_IS_HTML_CLASS(k)  (GTK_CHECK_CLASS_TYPE ((k), YELP_TYPE_HTML))
+#define YELP_HTML_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), YELP_TYPE_HTML, YelpHtmlClass))
 
 typedef struct _YelpHtml        YelpHtml;
 typedef struct _YelpHtmlClass   YelpHtmlClass;
@@ -46,19 +47,26 @@ struct _YelpHtml {
 struct _YelpHtmlClass {
         GObjectClass  parent_class;
 
+	guint font_handler;
+	guint color_handler;
+	guint a11y_handler;
+
 	/* Signals */
 	void (*uri_selected)   (YelpHtml    *view,
-				YelpURI     *uri,
+				gchar       *uri,
 				gboolean     handled);
 	void (*title_changed)  (YelpHtml    *view,
 				const gchar *new_title);
+	void (*popupmenu_requested) (YelpHtml *view,
+				     const gchar *link);
+
 };
 
 GType           yelp_html_get_type       (void);
 YelpHtml *      yelp_html_new            (void);
 
 void            yelp_html_set_base_uri   (YelpHtml    *html,
-					  YelpURI     *uri);
+					  const gchar *uri);
 void            yelp_html_clear          (YelpHtml    *html);
 void            yelp_html_write          (YelpHtml    *html,
 					  const gchar *data,
@@ -78,5 +86,7 @@ gboolean        yelp_html_find           (YelpHtml    *html,
 void            yelp_html_jump_to_anchor (YelpHtml    *html,
 					  gchar       *anchor);
 
+void            yelp_html_copy_selection (YelpHtml    *html);
+
 #endif /* __YELP_HTML_H__ */
 
